-- Hyperbolic Trig Functions and Natural Logarithms for Euphoria
-- Brad Olson, Jan 2003

include listly.e

-------------------------------------------------------------------------------------------------
-- NATURAL LOGARITHM FUNCTIONS
-- Euler  - the mathematical constant, e, Euler's Number
-- exp(x) - returns the natural logarithm of a sequence or atom
-------------------------------------------------------------------------------------------------

global constant Euler = 2.7182818284590451 -- the famous "e" of natural logarithms, Euler's Number

function exp( object n)
--E to the nth, properly handling lists and sublists
  return power( Euler, n)
end function

--- integer InE1ID
--- InE1ID=routine_id("InE1")

--- global function exp( object x)
--- --E to the Nth, recursively handling lists
---   return every( InE1ID, x)
--- end function

-------------------------------------------------------------------------------------------------
-- HYPERBOLIC TRIG FUNCTIONS
-- Thanks to www.sosmath.com for the basis of these formulae
-------------------------------------------------------------------------------------------------


------------------- sinh(n) arcsinh: hyperbolic sin in radians and its inverse

function sinh1( object radians)
--hyperbolic sin, atomic version
  return (exp( radians) - exp(-radians)) / 2
end function

function arcsinh1( object x)
--inverse hyperbolic sin, atomic version
  return log( x + sqrt( x * x + 1))
end function

constant idsinh1 = routine_id( "sinh1")
constant idarcsinh1=routine_id( "arcsinh1")

global function sinh( object radians)
  return every( idsinh1, radians)
end function  

global function arcsinh( object radians)
  return every( idarcsinh1, radians)
end function  



------------------- cosh(n) and arcosh: hyperbolic cosin in radians and its inverse

function cosh1( object radians)
--hyperbolic cosin, helper (atomic) version
  return (exp(radians) + exp(-radians)) / 2
end function

function arccosh1( object x)
--inverse hyperbolic cosin, atomic version
--(works on lists, but meant for atoms
--constrained to and result >= 0
--cosh is always positive, so e.g. cosh(3)=cosh(-3)
--I suppose we could return {-3,3} as the result, but could make for rank problems
  if x < 0 then return arccosh1( -x)
  else return log( x + sqrt( x * x - 1))
  end if
end function

constant idcosh1=routine_id("cosh1")
constant idarccosh1=routine_id("arccosh1")

global function cosh( object radians)
--hyperbolic cosin, optimized for lists and sublists
  return every( idcosh1, radians)
end function

global function arccosh( object x)
--hyperbolic cosin, optimized for lists and sublists
  return every( idarccosh1, x)
end function

------------------- tanh(n) and arctanh: hyperbolic tangent in radians and its inverse

function tanh1( object radians)
--hyperbolic tangent, atomic version
  object a
  a = exp( radians * 2)
  return (a-1) / (a+1)
end function  

function arctanh1( object x)
--inverse hyperbolic tangent, atomic version
  return log((1+x)/(1-x)) / 2
end function

constant idtanh1 = routine_id( "tanh1")
constant idarctanh1 = routine_id("arctanh1")

global function tanh( object radians)
--hyperbolic tangent, optimized for lists and sublists
  return every( idtanh1, radians)
end function  

global function arctanh( object x)
--hyperbolic tangent, optimized for lists and sublists
  return every( idarctanh1, x)
end function  

------------------- coth(n) and arcoth(n):hyperbolic cotangent in radians and its inverse  

global function coth( object radians)
--hyperbolic cotangent, error if 0
  return cosh(radians) / sinh(radians)
end function

global function arccoth( object x)
--the formula gets a divide by 0 error sometimes, have to fix that case where x=1
  return log((x+1)/(x-1))/2
end function  

------------------- sech(n) and csch(n)
global function sech( object radians)
--hyperbolic secant
  return 1 / cosh(radians)
end function

global function csch( object radians)
--hyperbolic cosecant
  return 1 / sinh(radians)
end function
